﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CatanGame.Model
{
    /// <summary>
    /// Represents a field on the board.
    /// </summary>
    public class Field
    {
        private ResourceType _resourceType;
        private Node[] _neighboringNodes;
        private int _fieldNum;

        /// <summary>
        /// Gets or sets the resource type on the field.
        /// </summary>
        public ResourceType ResourceType { get { return _resourceType; } set { _resourceType = value; } }
        /// <summary>
        /// Gets or sets the id of the field.
        /// </summary>
        public Byte FieldID { get; set; }
        /// <summary>
        /// Gets or sets the Number associated with the field.
        /// </summary>
        public int FieldNum { get { return _fieldNum; } set { _fieldNum = value; } }

        public Field()
        {

        }
        public Field(List<Node> neighbors)
        {
            _neighboringNodes = neighbors.ToArray();
        }

        /// <summary>
        /// Sends the resources to adjacent players.
        /// </summary>
        public void SendResources()
        {
            if (_resourceType != ResourceType.None && _resourceType != ResourceType.Sea)
                for (int i = 0; i < _neighboringNodes.Length; i++)
                {
                    if (_neighboringNodes[i].Building == null)
                        continue;
                    if (_neighboringNodes[i].Building.Type == HouseType.Settlement)
                        _neighboringNodes[i].Building.Owner.Resources[_resourceType] += 1;
                    else if (_neighboringNodes[i].Building.Type == HouseType.City)
                        _neighboringNodes[i].Building.Owner.Resources[_resourceType] += 2;
                }
        }
    }
}
