﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CatanGame.Model
{
    /// <summary>
    /// Represents the coordinates of a node.
    /// </summary>
    public struct NodeCoordinates : IEquatable<NodeCoordinates>
    {
        /// <summary>
        /// Gets the R axial coordinate.
        /// </summary>
        public int R { get; private set; }
        /// <summary>
        /// Gets the Q axial cordinate.
        /// </summary>
        public int Q { get; private set; }
        /// <summary>
        /// Gets the Direction(top, bottom) axial coordinate.
        /// </summary>
        public Direction Dir { get; private set; }

        public NodeCoordinates(int r, int q, Direction dir)
        {
            R = r;
            Q = q;
            Dir = dir;
        }

        public NodeCoordinates(NodeCoordinates other)
        {
            R = other.R;
            Q = other.Q;
            Dir = other.Dir;
        }

        /// <summary>
        /// Checks whether this coordinate is equal to the other.
        /// </summary>
        /// <param name="other">The other coordinate.</param>
        /// <returns>True if they are equal, false otherwise.</returns>
        public bool Equals(NodeCoordinates other)
        {
            return R == other.R && Q == other.Q && Dir == other.Dir;
        }

        /// <summary>
        /// Converts NodeCoordinates into strings.
        /// </summary>
        /// <returns>The formatted string.</returns>
        public override String ToString()
        {
            return R + "," + Q + "," + Dir.ToString();
        }
    }
}
