﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CatanGame.Model
{
    /// <summary>
    /// Represents the roads in the board.
    /// </summary>
    public class RoadMatrix
    {
        private List<Road>[,,] _roads;

        public RoadMatrix(int boardSize)
        {
            _roads = new List<Road>[boardSize, boardSize, 2];
            for (int q = 0; q < boardSize; ++q)
            {
                for (int r = 0; r < boardSize; ++r)
                {
                    _roads[r, q, 0] = new();
                    _roads[r, q, 1] = new();
                }
            }
        }

        /// <summary>
        /// Adds a road to the matrix.
        /// </summary>
        /// <param name="coords">The coordinates of the road.</param>
        /// <param name="playerId">The owner of the Road.</param>
        public void Add(RoadCoordinates coords, int playerId)
        {
            _roads[coords.From.R, coords.From.Q, (int)coords.From.Dir]
                .Add(new Road(new RoadCoordinates(coords.From, coords.To), playerId));
            _roads[coords.To.R, coords.To.Q, (int)coords.To.Dir]
                .Add(new Road(new RoadCoordinates(coords.To, coords.From), playerId));
        }

        /// <summary>
        /// Gets the roads coming out of given node.
        /// </summary>
        /// <param name="coords">The coordinates of the node.</param>
        /// <returns></returns>
        public List<Road> RoadsFromNode(NodeCoordinates coords)
        {
            return _roads[coords.R, coords.Q, (int)coords.Dir];
        }

        /// <summary>
        /// Decides whether a player has placed a road to the given coordinates.
        /// </summary>
        /// <param name="coords">The coordinates of the road.</param>
        /// <returns>True of a road is already there, false otherwise.</returns>
        public bool IsPlacedRoad(RoadCoordinates coords)
        {
            // the matrix is symmetric, so we need to search only in one direction
            return RoadsFromNode(coords.From)
                .Exists((road) => road.Coordinates.To.Equals(coords.To));
        }

    }
}
