using Microsoft.VisualStudio.TestTools.UnitTesting;
using CatanGame.Model;
using System.Collections.Generic;
using System;

namespace CatanGame.Test
{
    [TestClass]
    public class CatanGameModelTest
    {
        private GameModel model;
        private Random rand = new Random();

        public int GetNodeId(int r, int q, Direction dir)
        {
            return r * model.Board.Size * 2 + q * 2 + (int) dir;
        }

        public int GetNodeId(NodeCoordinates coords)
        {
            return GetNodeId(coords.R, coords.Q, coords.Dir);
        }

        [TestInitialize]
        public void Initialize()
        {
            CatanBoard.Rand = rand;
            model = new();
            model.NewGame();

        }

        /// <summary>
        /// road placement is successful test
        /// </summary>
        [TestMethod]
        public void RoadPlaceSuccessfullyTest()
        {
            model.UpgradeNode(new NodeCoordinates(3, 3, Direction.Top), 0);
            model.Board.Players[0].Resources[ResourceType.Brick] = 1;
            model.Board.Players[0].Resources[ResourceType.Lumber] = 1;
            NodeCoordinates from = new NodeCoordinates(3, 3, Direction.Top);
            NodeCoordinates to = new NodeCoordinates(1, 4, Direction.Bottom);
            RoadCoordinates road = new RoadCoordinates(from, to);
            model.PlaceRoad((GetNodeId(from), GetNodeId(to)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(road));
        }

        /// <summary>
        /// max road placement test
        /// </summary>
        [TestMethod]
        public void RoadPlaceMaxTest()
        {
            model.UpgradeNode(new NodeCoordinates(3, 3, Direction.Top), 0);
            model.PlaceRoad((GetNodeId(2, 4, Direction.Bottom), GetNodeId(3, 3, Direction.Top)));

            for (int i = 0; i < 6; i++)
                model.StepGame();
            model.Board.Players[0].Resources[ResourceType.Lumber] = 100;
            model.Board.Players[0].Resources[ResourceType.Brick] = 100;
            model.Board.Players[0].Resources[ResourceType.Grain] = 100;
            model.Board.Players[0].Resources[ResourceType.Ore] = 100;
            model.Board.Players[0].Resources[ResourceType.Wool] = 100;

            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(2, 4, Direction.Bottom), new NodeCoordinates(3, 3, Direction.Top))));
            model.PlaceRoad((GetNodeId(2, 4, Direction.Bottom), GetNodeId(4, 3, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(2, 4, Direction.Bottom), new NodeCoordinates(4, 3, Direction.Top))));
            model.PlaceRoad((GetNodeId(3, 3, Direction.Bottom), GetNodeId(4, 3, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(3, 3, Direction.Bottom), new NodeCoordinates(4, 3, Direction.Top))));
            model.PlaceRoad((GetNodeId(3, 3, Direction.Bottom), GetNodeId(4, 2, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(3, 3, Direction.Bottom), new NodeCoordinates(4, 2, Direction.Top))));
            model.PlaceRoad((GetNodeId(2, 3, Direction.Bottom), GetNodeId(4, 2, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(2, 3, Direction.Bottom), new NodeCoordinates(4, 2, Direction.Top))));
            model.PlaceRoad((GetNodeId(2, 3, Direction.Bottom), GetNodeId(3, 3, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(2, 3, Direction.Bottom), new NodeCoordinates(3, 3, Direction.Top))));
            model.PlaceRoad((GetNodeId(4, 3, Direction.Top), GetNodeId(3, 4, Direction.Bottom)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(4, 3, Direction.Top), new NodeCoordinates(3, 4, Direction.Bottom))));
            model.PlaceRoad((GetNodeId(4, 4, Direction.Top), GetNodeId(3, 4, Direction.Bottom)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(4, 4, Direction.Top), new NodeCoordinates(3, 4, Direction.Bottom))));
            model.PlaceRoad((GetNodeId(3, 3, Direction.Top), GetNodeId(1, 4, Direction.Bottom)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(3, 3, Direction.Top), new NodeCoordinates(1, 4, Direction.Bottom))));
            model.PlaceRoad((GetNodeId(2, 3, Direction.Top), GetNodeId(1, 4, Direction.Bottom)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(2, 3, Direction.Top), new NodeCoordinates(1, 4, Direction.Bottom))));
            model.PlaceRoad((GetNodeId(2, 3, Direction.Top), GetNodeId(1, 3, Direction.Bottom)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(2, 3, Direction.Top), new NodeCoordinates(1, 3, Direction.Bottom))));
            model.PlaceRoad((GetNodeId(2, 2, Direction.Top), GetNodeId(1, 3, Direction.Bottom)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(2, 2, Direction.Top), new NodeCoordinates(1, 3, Direction.Bottom))));
            model.PlaceRoad((GetNodeId(2, 2, Direction.Top), GetNodeId(0, 3, Direction.Bottom)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(2, 2, Direction.Top), new NodeCoordinates(0, 3, Direction.Bottom))));
            model.PlaceRoad((GetNodeId(1, 3, Direction.Top), GetNodeId(0, 3, Direction.Bottom)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(1, 3, Direction.Top), new NodeCoordinates(0, 3, Direction.Bottom))));
            model.PlaceRoad((GetNodeId(1, 3, Direction.Top), GetNodeId(0, 4, Direction.Bottom)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(1, 3, Direction.Top), new NodeCoordinates(0, 4, Direction.Bottom))));
            model.PlaceRoad((GetNodeId(1, 4, Direction.Top), GetNodeId(0, 4, Direction.Bottom)));
            Assert.IsFalse(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(1, 4, Direction.Top), new NodeCoordinates(0, 4, Direction.Bottom))));
        }

        /// <summary>
        /// another max road placement test
        /// </summary>
        [TestMethod]
        public void RoadPlaceMaxTest2()
        {
            CatanBoard.Rand = new RandomMock(new List<int>() { 3, 6 });

            model.UpgradeNode(new NodeCoordinates(3, 3, Direction.Top), 0);
            model.PlaceRoad((GetNodeId(2, 4, Direction.Bottom), GetNodeId(3, 3, Direction.Top)));

            model.UpgradeNode(new NodeCoordinates(2, 5, Direction.Top), 1);
            Assert.AreEqual(model.CurrentPlayerId, 1);
            model.PlaceRoad((GetNodeId(2, 5, Direction.Top), GetNodeId(1, 5, Direction.Bottom)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(2, 5, Direction.Top), new NodeCoordinates(1, 5, Direction.Bottom))));

            for (int i = 0; i < 6; i++)
                model.StepGame();
            model.Roll();
            model.Board.Players[0].Resources[ResourceType.Lumber] = 100;
            model.Board.Players[0].Resources[ResourceType.Brick] = 100;
            model.Board.Players[0].Resources[ResourceType.Grain] = 100;
            model.Board.Players[0].Resources[ResourceType.Ore] = 100;
            model.Board.Players[0].Resources[ResourceType.Wool] = 100;
            model.Board.Players[1].Resources[ResourceType.Lumber] = 100;
            model.Board.Players[1].Resources[ResourceType.Brick] = 100;
            model.Board.Players[1].Resources[ResourceType.Grain] = 100;
            model.Board.Players[1].Resources[ResourceType.Ore] = 100;
            model.Board.Players[1].Resources[ResourceType.Wool] = 100;

            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(2, 4, Direction.Bottom), new NodeCoordinates(3, 3, Direction.Top))));
            model.PlaceRoad((GetNodeId(2, 4, Direction.Bottom), GetNodeId(4, 3, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(2, 4, Direction.Bottom), new NodeCoordinates(4, 3, Direction.Top))));
            model.PlaceRoad((GetNodeId(3, 3, Direction.Bottom), GetNodeId(4, 3, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(3, 3, Direction.Bottom), new NodeCoordinates(4, 3, Direction.Top))));
            model.PlaceRoad((GetNodeId(3, 3, Direction.Bottom), GetNodeId(4, 2, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(3, 3, Direction.Bottom), new NodeCoordinates(4, 2, Direction.Top))));
            model.PlaceRoad((GetNodeId(2, 3, Direction.Bottom), GetNodeId(4, 2, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(2, 3, Direction.Bottom), new NodeCoordinates(4, 2, Direction.Top))));
            model.PlaceRoad((GetNodeId(2, 3, Direction.Bottom), GetNodeId(3, 3, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(2, 3, Direction.Bottom), new NodeCoordinates(3, 3, Direction.Top))));
            model.PlaceRoad((GetNodeId(4, 3, Direction.Top), GetNodeId(3, 4, Direction.Bottom)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(4, 3, Direction.Top), new NodeCoordinates(3, 4, Direction.Bottom))));
            model.PlaceRoad((GetNodeId(4, 4, Direction.Top), GetNodeId(3, 4, Direction.Bottom)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(4, 4, Direction.Top), new NodeCoordinates(3, 4, Direction.Bottom))));
            model.PlaceRoad((GetNodeId(3, 3, Direction.Top), GetNodeId(1, 4, Direction.Bottom)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(3, 3, Direction.Top), new NodeCoordinates(1, 4, Direction.Bottom))));
            model.PlaceRoad((GetNodeId(2, 3, Direction.Top), GetNodeId(1, 4, Direction.Bottom)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(2, 3, Direction.Top), new NodeCoordinates(1, 4, Direction.Bottom))));
            model.PlaceRoad((GetNodeId(2, 3, Direction.Top), GetNodeId(1, 3, Direction.Bottom)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(2, 3, Direction.Top), new NodeCoordinates(1, 3, Direction.Bottom))));
            model.PlaceRoad((GetNodeId(2, 2, Direction.Top), GetNodeId(1, 3, Direction.Bottom)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(2, 2, Direction.Top), new NodeCoordinates(1, 3, Direction.Bottom))));
            model.PlaceRoad((GetNodeId(2, 2, Direction.Top), GetNodeId(0, 3, Direction.Bottom)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(2, 2, Direction.Top), new NodeCoordinates(0, 3, Direction.Bottom))));
            model.PlaceRoad((GetNodeId(1, 3, Direction.Top), GetNodeId(0, 3, Direction.Bottom)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(1, 3, Direction.Top), new NodeCoordinates(0, 3, Direction.Bottom))));
            model.PlaceRoad((GetNodeId(1, 3, Direction.Top), GetNodeId(0, 4, Direction.Bottom)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(1, 3, Direction.Top), new NodeCoordinates(0, 4, Direction.Bottom))));
            model.PlaceRoad((GetNodeId(1, 4, Direction.Top), GetNodeId(0, 4, Direction.Bottom)));
            Assert.IsFalse(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(1, 4, Direction.Top), new NodeCoordinates(0, 4, Direction.Bottom))));

            model.StepGame();
            model.Roll();
            Assert.AreEqual(model.CurrentPlayerId, 1);
            model.PlaceRoad((GetNodeId(2, 5, Direction.Top), GetNodeId(1, 6, Direction.Bottom)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(2, 5, Direction.Top), new NodeCoordinates(1, 6, Direction.Bottom))));
        }

        /// <summary>
        /// road placement without settlement
        /// </summary>
        [TestMethod]
        public void RoadPlaceWithoutSettlementTest1()
        {
            model.Board.Players[0].Resources[ResourceType.Brick] = 1;
            model.Board.Players[0].Resources[ResourceType.Lumber] = 1;
            NodeCoordinates from = new NodeCoordinates(3, 3, Direction.Top);
            NodeCoordinates to = new NodeCoordinates(1, 4, Direction.Bottom);
            RoadCoordinates road = new RoadCoordinates(from, to);
            model.PlaceRoad((GetNodeId(from), GetNodeId(to)));
            Assert.IsFalse(model.Board.Roads.IsPlacedRoad(road));
        }

        /// <summary>
        /// another road placement without settlement test
        /// </summary>
        [TestMethod]
        public void RoadPlaceWithoutSettlementTest2()
        {
            for (int i = 0; i < 6; i++)
                model.StepGame();
            model.UpgradeNode(new NodeCoordinates(3, 3, Direction.Top), 1);
            model.Board.Players[0].Resources[ResourceType.Brick] = 1;
            model.Board.Players[0].Resources[ResourceType.Lumber] = 1;
            NodeCoordinates from = new NodeCoordinates(3, 3, Direction.Top);
            NodeCoordinates to = new NodeCoordinates(1, 4, Direction.Bottom);
            RoadCoordinates road = new RoadCoordinates(from, to);
            model.PlaceRoad((GetNodeId(from), GetNodeId(to)));
            Assert.IsFalse(model.Board.Roads.IsPlacedRoad(road));
        }

        /// <summary>
        /// road placement without resources test
        /// </summary>
        [TestMethod]
        public void RoadPlaceWithoutResourcesTest()
        {
            for (int i = 0; i < 6; i++)
                model.StepGame();
            model.UpgradeNode(new NodeCoordinates(3, 3, Direction.Top), 0);
            model.Board.Players[0].Resources[ResourceType.Brick] = 0;
            model.Board.Players[0].Resources[ResourceType.Lumber] = 1;
            NodeCoordinates from = new NodeCoordinates(3, 3, Direction.Top);
            NodeCoordinates to = new NodeCoordinates(1, 4, Direction.Bottom);
            RoadCoordinates road = new RoadCoordinates(from, to);
            model.PlaceRoad((GetNodeId(from), GetNodeId(to)));
            Assert.IsFalse(model.Board.Roads.IsPlacedRoad(road));
        }

        /// <summary>
        /// road placement next to placed road
        /// </summary>
        [TestMethod]
        public void RoadPlaceNextToPlacedRoadTest()
        {
            model.UpgradeNode(new NodeCoordinates(3, 3, Direction.Top), 0);
            model.Board.Players[0].Resources[ResourceType.Brick] = 1;
            model.Board.Players[0].Resources[ResourceType.Lumber] = 1;
            NodeCoordinates from = new NodeCoordinates(3, 3, Direction.Top);
            NodeCoordinates to = new NodeCoordinates(1, 4, Direction.Bottom);
            RoadCoordinates road = new RoadCoordinates(from, to);
            model.PlaceRoad((GetNodeId(from), GetNodeId(to)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(road));
            for (int i = 0; i < 6; i++)
                model.StepGame();
            NodeCoordinates from2 = to;
            NodeCoordinates to2 = new NodeCoordinates(2, 4, Direction.Top);
            RoadCoordinates road2 = new RoadCoordinates(from2, to2);
            model.PlaceRoad((GetNodeId(from2), GetNodeId(to2)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(road2));
        }

        /// <summary>
        /// road placement with illegal coords test
        /// </summary>
        [TestMethod]
        public void RoadPlaceWithIllegalCoordsTest()
        {
            model.UpgradeNode(new NodeCoordinates(3, 3, Direction.Top), 0);
            model.Board.Players[0].Resources[ResourceType.Brick] = 1;
            model.Board.Players[0].Resources[ResourceType.Lumber] = 1;
            NodeCoordinates from = new NodeCoordinates(3, 3, Direction.Top);
            NodeCoordinates to = new NodeCoordinates(1, 4, Direction.Bottom);
            RoadCoordinates road = new RoadCoordinates(from, to);
            model.PlaceRoad((GetNodeId(from), GetNodeId(to)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(road));
            for (int i = 0; i < 6; i++)
                model.StepGame();
            NodeCoordinates from2 = to;
            NodeCoordinates to2 = new NodeCoordinates(2, 5, Direction.Top);
            RoadCoordinates road2 = new RoadCoordinates(from2, to2);
            model.PlaceRoad((GetNodeId(from2), GetNodeId(to2)));
            Assert.IsFalse(model.Board.Roads.IsPlacedRoad(road2));
        }

        /// <summary>
        /// road placement next to placed road test 2
        /// </summary>
        [TestMethod]
        public void RoadPlaceNotNextToPlacedRoadTest()
        {
            model.UpgradeNode(new NodeCoordinates(3, 3, Direction.Top), 0);
            model.Board.Players[0].Resources[ResourceType.Brick] = 1;
            model.Board.Players[0].Resources[ResourceType.Lumber] = 1;
            NodeCoordinates from = new NodeCoordinates(3, 3, Direction.Top);
            NodeCoordinates to = new NodeCoordinates(1, 4, Direction.Bottom);
            RoadCoordinates road = new RoadCoordinates(from, to);
            model.PlaceRoad((GetNodeId(from), GetNodeId(to)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(road));
            for (int i = 0; i < 6; i++)
                model.StepGame();
            NodeCoordinates from2 = new NodeCoordinates(3, 3, Direction.Bottom);
            NodeCoordinates to2 = new NodeCoordinates(2, 5, Direction.Top);
            RoadCoordinates road2 = new RoadCoordinates(from2, to2);
            model.PlaceRoad((GetNodeId(from2), GetNodeId(to2)));
            Assert.IsFalse(model.Board.Roads.IsPlacedRoad(road2));
        }

        /// <summary>
        /// game step test
        /// </summary>
        [TestMethod]
        public void StepGameTest()
        {
            CatanBoard.Rand = new RandomMock(new List<int>() { 1 });
            Assert.AreEqual(0, model.CurrentPlayerId);
            model.StepGame();
            Assert.AreEqual(1, model.CurrentPlayerId);
            model.StepGame();
            Assert.AreEqual(2, model.CurrentPlayerId);
            model.StepGame();
            Assert.AreEqual(2, model.CurrentPlayerId);
            model.StepGame();
            Assert.AreEqual(1, model.CurrentPlayerId);
            model.StepGame();
            Assert.AreEqual(0, model.CurrentPlayerId);
            model.StepGame();
            Assert.AreEqual(0, model.CurrentPlayerId);
            model.Roll();
            model.StepGame();
            Assert.AreEqual(1, model.CurrentPlayerId);
        }

        /// <summary>
        /// send resource to the players test
        /// </summary>
        [TestMethod]
        public void SendResourcesTest()
        {
            model.Board.SendResources();
            model.UpgradeNode(new NodeCoordinates(3, 3, Direction.Top), 0);
            int n = 0;
            foreach (int cnt in model.Board.CurrentPlayer.Resources.Values)
                n += cnt;
            Assert.AreEqual(n, 0);
            model.Board.SendResources();
            foreach (int cnt in model.Board.CurrentPlayer.Resources.Values)
                n += cnt;
            Assert.IsTrue(n > 1);
            Assert.IsTrue(n < 4);
        }

        /// <summary>
        /// send resource to the players with robber test
        /// </summary>
        [TestMethod]
        public void SendResourcesWithRobberTest()
        {
            CatanBoard.Rand = new RandomMock(new List<int>() { 3, 4 });

            model.Board.SendResources();
            model.UpgradeNode(new NodeCoordinates(2, 3, Direction.Top), 0);
            int n = 0;
            foreach (int cnt in model.Board.CurrentPlayer.Resources.Values)
                n += cnt;
            Assert.AreEqual(n, 0);

            for (int i = 0; i < 6; i++)
                model.StepGame();

            foreach (int cnt in model.Board.CurrentPlayer.Resources.Values)
                n += cnt;

            model.Roll();
            model.MoveRobber((2, 3));
            Assert.AreEqual(model.Board.RobberPosition, (2, 3));

            int k = 0;
            model.Board.SendResources();
            foreach (int cnt in model.Board.CurrentPlayer.Resources.Values)
                k += cnt;
            Assert.IsTrue(n < k);
        }

        /// <summary>
        /// current player trade test
        /// </summary>
        [TestMethod]
        public void TradeTest()
        {
            // nincs eleg eroforras
            Assert.IsFalse(model.Trade(new List<(ResourceType, int)>() { (ResourceType.Brick, 3) }, new List<(ResourceType, int)>() { (ResourceType.Ore, 1) }));
            model.Board.Players[0].Resources[ResourceType.Brick] = 1;
            Assert.IsFalse(model.Trade(new List<(ResourceType, int)>() { (ResourceType.Brick, 3) }, new List<(ResourceType, int)>() { (ResourceType.Ore, 1) }));
            Assert.AreEqual(model.Board.Players[0].Resources[ResourceType.Brick], 1);
            Assert.AreEqual(model.Board.Players[0].Resources[ResourceType.Ore], 0);
            
            // van eleg eroforras
            model.Board.Players[0].Resources[ResourceType.Brick] = 3;
            Assert.IsTrue(model.Trade(new List<(ResourceType, int)>() { (ResourceType.Brick, 3) }, new List<(ResourceType, int)>() { (ResourceType.Ore, 1) }));
            Assert.AreEqual(model.Board.Players[0].Resources[ResourceType.Ore], 1);
            Assert.AreEqual(model.Board.Players[0].Resources[ResourceType.Brick], 0);
        }

        /// <summary>
        /// current player enable roads test
        /// </summary>
        [TestMethod]
        public void EnableRoadsTest()
        {
            model.UpgradeNode(new NodeCoordinates(3, 3, Direction.Bottom), 0);
            List<RoadCoordinates> enableRoads = model.Board.EnabledRoads(model.Board.Players[0]);
            List<RoadCoordinates> enableRoadsT = new();
            enableRoadsT.Add(new RoadCoordinates(new NodeCoordinates(3, 3, Direction.Bottom), new NodeCoordinates(4, 3, Direction.Top)));
            enableRoadsT.Add(new RoadCoordinates(new NodeCoordinates(3, 3, Direction.Bottom), new NodeCoordinates(4, 2, Direction.Top)));
            enableRoadsT.Add(new RoadCoordinates(new NodeCoordinates(3, 3, Direction.Bottom), new NodeCoordinates(5, 2, Direction.Top)));
            
            foreach (RoadCoordinates road in enableRoadsT)
                Assert.IsTrue(enableRoads.Contains(road));
        }

        /// <summary>
        ///  current player enable roads wit one road test
        /// </summary>
        [TestMethod]
        public void EnableRoadsTestWithRoads()
        {
            model.UpgradeNode(new NodeCoordinates(3, 3, Direction.Bottom), 0);
            model.PlaceRoad((GetNodeId(3, 3, Direction.Bottom), GetNodeId(4, 3, Direction.Top)));
            List<RoadCoordinates> enableRoads = model.Board.EnabledRoads(model.Board.Players[0]);
            List<RoadCoordinates> enableRoadsT = new();
            enableRoadsT.Add(new RoadCoordinates(new NodeCoordinates(2, 4, Direction.Bottom), new NodeCoordinates(4, 3, Direction.Top)));
            enableRoadsT.Add(new RoadCoordinates(new NodeCoordinates(3, 4, Direction.Bottom), new NodeCoordinates(4, 3, Direction.Top)));
            enableRoadsT.Add(new RoadCoordinates(new NodeCoordinates(3, 3, Direction.Bottom), new NodeCoordinates(4, 2, Direction.Top)));
            enableRoadsT.Add(new RoadCoordinates(new NodeCoordinates(3, 3, Direction.Bottom), new NodeCoordinates(5, 2, Direction.Top)));

            foreach (RoadCoordinates road in enableRoadsT)
                Assert.IsTrue(enableRoads.Contains(road));
        }

        /// <summary>
        /// Enable Roads With Someone Elses Settlemen test
        /// </summary>
        [TestMethod]
        public void EnableRoadsTestWithSomeoneElsesSettlement()
        {
            model.UpgradeNode(new NodeCoordinates(3, 3, Direction.Bottom), 0);
            model.PlaceRoad((GetNodeId(3, 3, Direction.Bottom), GetNodeId(4, 3, Direction.Top)));
            model.UpgradeNode(new NodeCoordinates(2, 4, Direction.Bottom), 1);
            List<RoadCoordinates> enableRoads = model.Board.EnabledRoads(model.Board.Players[0]);
            List<RoadCoordinates> enableRoadsT = new();
            enableRoadsT.Add(new RoadCoordinates(new NodeCoordinates(2, 4, Direction.Bottom), new NodeCoordinates(4, 3, Direction.Top)));
            enableRoadsT.Add(new RoadCoordinates(new NodeCoordinates(3, 4, Direction.Bottom), new NodeCoordinates(4, 3, Direction.Top)));
            enableRoadsT.Add(new RoadCoordinates(new NodeCoordinates(3, 3, Direction.Bottom), new NodeCoordinates(4, 2, Direction.Top)));
            enableRoadsT.Add(new RoadCoordinates(new NodeCoordinates(3, 3, Direction.Bottom), new NodeCoordinates(5, 2, Direction.Top)));

            foreach (RoadCoordinates road in enableRoadsT)
                Assert.IsTrue(enableRoads.Contains(road));
        }

        /// <summary>
        /// another Enable Roads With Someone Elses Settlemen test
        /// </summary>
        [TestMethod]
        public void EnableRoadsTestWithSomeoneElsesSettlement2()
        {
            model.UpgradeNode(new NodeCoordinates(3, 3, Direction.Bottom), 0);
            model.PlaceRoad((GetNodeId(3, 3, Direction.Bottom), GetNodeId(4, 3, Direction.Top)));
            model.PlaceRoad((GetNodeId(2, 4, Direction.Bottom), GetNodeId(4, 3, Direction.Top)));
            model.UpgradeNode(new NodeCoordinates(2, 4, Direction.Bottom), 1);
            List<RoadCoordinates> enableRoads = model.Board.EnabledRoads(model.Board.Players[0]);
            List<RoadCoordinates> enableRoadsT = new();
            enableRoadsT.Add(new RoadCoordinates(new NodeCoordinates(2, 4, Direction.Bottom), new NodeCoordinates(4, 3, Direction.Top)));
            enableRoadsT.Add(new RoadCoordinates(new NodeCoordinates(3, 4, Direction.Bottom), new NodeCoordinates(4, 3, Direction.Top)));
            enableRoadsT.Add(new RoadCoordinates(new NodeCoordinates(3, 3, Direction.Bottom), new NodeCoordinates(4, 2, Direction.Top)));
            enableRoadsT.Add(new RoadCoordinates(new NodeCoordinates(3, 3, Direction.Bottom), new NodeCoordinates(5, 2, Direction.Top)));

            foreach (RoadCoordinates road in enableRoadsT)
                Assert.IsTrue(enableRoads.Contains(road));
        }

        /// <summary>
        /// Enable Roads With Someone Elses Road test
        /// </summary>
        [TestMethod]
        public void EnableRoadsTestWithSomeoneElsesRoad()
        {
            model.UpgradeNode(new NodeCoordinates(3, 3, Direction.Bottom), 0);
            model.PlaceRoad((GetNodeId(3, 3, Direction.Bottom), GetNodeId(4, 3, Direction.Top)));
            model.StepGame();
            model.PlaceRoad((GetNodeId(2, 4, Direction.Bottom), GetNodeId(4, 3, Direction.Top)));
            model.StepGame();
            model.StepGame();
            model.UpgradeNode(new NodeCoordinates(2, 4, Direction.Bottom), 1);
            List<RoadCoordinates> enableRoads = model.Board.EnabledRoads(model.Board.Players[0]);
            List<RoadCoordinates> enableRoadsT = new();
            enableRoadsT.Add(new RoadCoordinates(new NodeCoordinates(3, 4, Direction.Bottom), new NodeCoordinates(4, 3, Direction.Top)));
            enableRoadsT.Add(new RoadCoordinates(new NodeCoordinates(3, 3, Direction.Bottom), new NodeCoordinates(4, 2, Direction.Top)));
            enableRoadsT.Add(new RoadCoordinates(new NodeCoordinates(3, 3, Direction.Bottom), new NodeCoordinates(5, 2, Direction.Top)));

            foreach (RoadCoordinates road in enableRoadsT)
                Assert.IsTrue(enableRoads.Contains(road));
        }

        /// <summary>
        /// game not over test
        /// </summary>
        [TestMethod]
        public void GameOverTest()
        {
            Assert.IsFalse(model.IsGameOver());
        }

        /// <summary>
        /// another game not over test
        /// </summary>
        [TestMethod]
        public void GameOverTest2()
        {
            model.Board.Players[model.CurrentPlayerId].Score = 4;
            Assert.IsFalse(model.IsGameOver());
        }

        /// <summary>
        /// game over test
        /// </summary>
        [TestMethod]
        public void GameOverTest3()
        {
            model.Board.Players[model.CurrentPlayerId].Score = 5;
            Assert.IsTrue(model.IsGameOver());
        }

        /// <summary>
        /// another game over test
        /// </summary>
        [TestMethod]
        public void GameOverTest4()
        {
            model.Board.Players[model.CurrentPlayerId].Score = 6;
            Assert.IsTrue(model.IsGameOver());
        }

        /// <summary>
        /// house placement successfull test
        /// </summary>
        [TestMethod]
        public void SuccessfullyPlaceHouseTest()
        {
            model.Board.Players[0].Resources[ResourceType.Lumber] = 1;
            model.Board.Players[0].Resources[ResourceType.Grain] = 1;
            model.Board.Players[0].Resources[ResourceType.Wool] = 1;
            model.Board.Players[0].Resources[ResourceType.Brick] = 1;

            Assert.IsTrue(model.Board.PlaceSettlement(new NodeCoordinates(3, 3, Direction.Bottom), model.Board.Players[0]));

            for(int r = 0; r < model.Board.Size; ++r)
            {
                for (int q = 0; q < model.Board.Size; ++q)
                {
                    for (int dir = 0; dir < 2; ++dir)
                    {
                        if (r == 3 && q == 3 && dir == (int)Direction.Bottom)
                            Assert.AreEqual(model.Board.Nodes[r, q, (Direction)dir].Building.Type, HouseType.Settlement);
                        else
                            Assert.IsTrue(model.Board.Nodes[r, q, (Direction)dir] is null
                                || model.Board.Nodes[r, q, (Direction)dir].Building is null 
                                || model.Board.Nodes[r, q, (Direction)dir].Building.Type == HouseType.NoBuilding);
                            
                    }
                }
            }   
        }

        /// <summary>
        /// house placement without resources test
        /// </summary>
        [TestMethod]
        public void PlaceHouseWithoutResourcesTest()
        {
            Assert.IsTrue(model.Board.PlaceSettlement(new NodeCoordinates(3, 3, Direction.Bottom), model.Board.Players[0]));

            NodeCoordinates from = new NodeCoordinates(3, 3, Direction.Bottom);
            NodeCoordinates to = new NodeCoordinates(5, 2, Direction.Top);
            RoadCoordinates road = new RoadCoordinates(from, to);
            model.PlaceRoad((GetNodeId(from), GetNodeId(to)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(road));

            for (int i = 0; i < 6; i++)
                model.StepGame();

            model.Board.Players[0].Resources[ResourceType.Lumber] = 1;
            model.Board.Players[0].Resources[ResourceType.Grain] = 1;
            model.Board.Players[0].Resources[ResourceType.Wool] = 1;
            model.Board.Players[0].Resources[ResourceType.Brick] = 1;

            NodeCoordinates from2 = to;
            NodeCoordinates to2 = new NodeCoordinates(4, 3, Direction.Bottom);
            RoadCoordinates road2 = new RoadCoordinates(from2, to2);
            model.PlaceRoad((GetNodeId(from2), GetNodeId(to2)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(road2));

            Assert.IsFalse(model.Board.PlaceSettlement(new NodeCoordinates(4, 3, Direction.Bottom), model.Board.Players[0]));
        }

        /// <summary>
        /// house placement with resources test
        /// </summary>
        [TestMethod]
        public void PlaceHouseWithResourcesTest()
        {
            model.Board.Players[0].Resources[ResourceType.Lumber] = 2;
            model.Board.Players[0].Resources[ResourceType.Grain] = 1;
            model.Board.Players[0].Resources[ResourceType.Wool] = 1;
            model.Board.Players[0].Resources[ResourceType.Brick] = 2;

            Assert.IsTrue(model.Board.PlaceSettlement(new NodeCoordinates(3, 3, Direction.Bottom), model.Board.Players[0]));

            NodeCoordinates from = new NodeCoordinates(3, 3, Direction.Bottom);
            NodeCoordinates to = new NodeCoordinates(5, 2, Direction.Top);
            RoadCoordinates road = new RoadCoordinates(from, to);
            model.PlaceRoad((GetNodeId(from), GetNodeId(to)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(road));

            for (int i = 0; i < 6; i++)
                model.StepGame();

            NodeCoordinates from2 = to;
            NodeCoordinates to2 = new NodeCoordinates(4, 3, Direction.Bottom);
            RoadCoordinates road2 = new RoadCoordinates(from2, to2);
            model.PlaceRoad((GetNodeId(from2), GetNodeId(to2)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(road2));

            Assert.IsTrue(model.Board.PlaceSettlement(new NodeCoordinates(4, 3, Direction.Bottom), model.Board.Players[0]));
        }

        /// <summary>
        /// house placement next to placed building test
        /// </summary>
        [TestMethod]
        public void PlaceHouseNextToPlacedBuildingTest()
        {
            model.Board.Players[0].Resources[ResourceType.Lumber] = 1;
            model.Board.Players[0].Resources[ResourceType.Grain] = 1;
            model.Board.Players[0].Resources[ResourceType.Wool] = 1;
            model.Board.Players[0].Resources[ResourceType.Brick] = 1;

            Assert.IsTrue(model.Board.PlaceSettlement(new NodeCoordinates(3, 3, Direction.Bottom), model.Board.Players[0]));

            NodeCoordinates from = new NodeCoordinates(3, 3, Direction.Bottom);
            NodeCoordinates to = new NodeCoordinates(5, 2, Direction.Top);
            RoadCoordinates road = new RoadCoordinates(from, to);
            model.PlaceRoad((GetNodeId(from), GetNodeId(to)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(road));

            for (int i = 0; i < 6; i++)
                model.StepGame();

            Assert.IsFalse(model.Board.PlaceSettlement(new NodeCoordinates(5, 2, Direction.Bottom), model.Board.Players[0]));
        }

        /// <summary>
        /// house placement in the first round without road test
        /// </summary>
        [TestMethod]
        public void PlaceHouseNotNextToPlacedRoadAfterInitialTurnTest()
        {
            model.Board.Players[0].Resources[ResourceType.Lumber] = 1;
            model.Board.Players[0].Resources[ResourceType.Grain] = 1;
            model.Board.Players[0].Resources[ResourceType.Wool] = 1;
            model.Board.Players[0].Resources[ResourceType.Brick] = 1;

            Assert.IsTrue(model.Board.PlaceSettlement(new NodeCoordinates(3, 3, Direction.Bottom), model.Board.Players[0]));

            NodeCoordinates from = new NodeCoordinates(3, 3, Direction.Bottom);
            NodeCoordinates to = new NodeCoordinates(5, 2, Direction.Top);
            RoadCoordinates road = new RoadCoordinates(from, to);
            model.PlaceRoad((GetNodeId(from), GetNodeId(to)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(road));

            for (int i = 0; i < 6; i++)
                model.StepGame();

            Assert.IsFalse(model.Board.PlaceSettlement(new NodeCoordinates(1, 1, Direction.Bottom), model.Board.Players[0]));
        }

        /// <summary>
        /// city placement test
        /// </summary>
        [TestMethod]
        public void PlaceCityTest()
        {
            CatanBoard.Rand = new RandomMock(new List<int>() { 1, 2 });

            for (int i = 0; i < 5; i++)
                model.StepGame();

            model.Board.Players[0].Resources[ResourceType.Lumber] = 1;
            model.Board.Players[0].Resources[ResourceType.Grain] = 4;
            model.Board.Players[0].Resources[ResourceType.Wool] = 1;
            model.Board.Players[0].Resources[ResourceType.Brick] = 1;
            model.Board.Players[0].Resources[ResourceType.Ore] = 3;

            Assert.IsTrue(model.UpgradeNode(new NodeCoordinates(3, 3, Direction.Bottom), 0));
            model.StepGame();
            model.Roll();
            Assert.IsTrue(model.UpgradeNode(new NodeCoordinates(3, 3, Direction.Bottom), 0));

            for (int r = 0; r < model.Board.Size; ++r)
            {
                for (int q = 0; q < model.Board.Size; ++q)
                {
                    for (int dir = 0; dir < 2; ++dir)
                    {
                        if (r == 3 && q == 3 && dir == (int)Direction.Bottom)
                            Assert.AreEqual(model.Board.Nodes[r, q, (Direction)dir].Building.Type, HouseType.City);
                        else
                            Assert.IsTrue(model.Board.Nodes[r, q, (Direction)dir] is null
                                || model.Board.Nodes[r, q, (Direction)dir].Building is null
                                || model.Board.Nodes[r, q, (Direction)dir].Building.Type == HouseType.NoBuilding);

                    }
                }
            }
        }

        /// <summary>
        /// city placement without resources test
        /// </summary>
        [TestMethod]
        public void PlaceCityWithoutResourcesTest()
        {
            model.Board.Players[0].Resources[ResourceType.Lumber] = 1;
            model.Board.Players[0].Resources[ResourceType.Grain] = 2;
            model.Board.Players[0].Resources[ResourceType.Wool] = 1;
            model.Board.Players[0].Resources[ResourceType.Brick] = 1;
            model.Board.Players[0].Resources[ResourceType.Ore] = 3;

            Assert.IsTrue(model.UpgradeNode(new NodeCoordinates(3, 3, Direction.Bottom), 0));
            Assert.IsFalse(model.UpgradeNode(new NodeCoordinates(3, 3, Direction.Bottom), 0));

            for (int r = 0; r < model.Board.Size; ++r)
            {
                for (int q = 0; q < model.Board.Size; ++q)
                {
                    for (int dir = 0; dir < 2; ++dir)
                    {
                        if (r == 3 && q == 3 && dir == (int)Direction.Bottom)
                            Assert.AreEqual(model.Board.Nodes[r, q, (Direction)dir].Building.Type, HouseType.Settlement);
                        else
                            Assert.IsTrue(model.Board.Nodes[r, q, (Direction)dir] is null
                                || model.Board.Nodes[r, q, (Direction)dir].Building is null
                                || model.Board.Nodes[r, q, (Direction)dir].Building.Type == HouseType.NoBuilding);

                    }
                }
            }
        }

        /// <summary>
        /// show enable nodes in the first round test
        /// </summary>
        [TestMethod]
        public void ShowEnabledNodesTest()
        {
            List<NodeCoordinates> allnodes = new();
            for (int r = 0; r <model.Board.Size; r++)
                for (int q = 0; q < model.Board.Size; q++)
                    for (int d = 0; d < 2; d++)
                        if (model.Board.CanPlaceSettlement(new NodeCoordinates(r, q, (Direction)d), model.Board.Players[0]))
                            allnodes.Add(new NodeCoordinates(r, q, (Direction)d));
            List<NodeCoordinates> enableNodes = model.Board.EnableNodes(model.Board.Players[0]);
            foreach (NodeCoordinates node in allnodes)
                Assert.IsTrue(enableNodes.Contains(node));


        }

        /// <summary>
        /// show enable nodes after place one nodes test
        /// </summary>
        [TestMethod]
        public void ShowEnabledNodesAfterPlaceNodesTest()
        {
            model.Board.PlaceSettlement(new NodeCoordinates(3, 3, Direction.Bottom), model.Board.Players[0]);
            List<NodeCoordinates> enableNoodsT = new();
            for (int r = 0; r < model.Board.Size; r++)
                for (int q = 0; q < model.Board.Size; q++)
                    for (int d = 0; d < 2; d++)
                        if ((r != 3 && q != 3 && d != 0) || (r != 2 && q != 4 && d != 1) || (r != 2 && q != 5 && d != 1) || (r != 3 && q != 4 && d != 1))
                            if (model.Board.CanPlaceSettlement(new NodeCoordinates(r, q, (Direction)d), model.Board.Players[0]))
                                enableNoodsT.Add(new NodeCoordinates(r, q, (Direction)d));
            List<NodeCoordinates> enableNodes = model.Board.EnableNodes(model.Board.Players[0]);
            foreach (NodeCoordinates node in enableNoodsT)
                Assert.IsTrue(enableNodes.Contains(node));
        }

        /// <summary>
        /// buy bonus card test
        /// </summary>
        [TestMethod]
        public void BuyBonusCardTest()
        {
            CatanBoard.Rand = new RandomMock(new List<int>() { 1 });
            Assert.IsFalse(model.Board.Players[0].CanBuyDevelopmentCard);
            model.Board.Players[0].Resources[ResourceType.Grain] = 1;
            model.Board.Players[0].Resources[ResourceType.Wool] = 1;
            model.Board.Players[0].Resources[ResourceType.Ore] = 1;
            Assert.AreEqual(model.Board.BuyCard(model.Board.Players[0]), CardType.None);

            for (int i = 0; i < 6; i++)
            {
                model.StepGame();
            }
            model.Roll();

            Assert.IsTrue(model.Board.Players[0].CanBuyDevelopmentCard);
            Assert.AreNotEqual(model.Board.BuyCard(model.Board.Players[0]),CardType.None);
        }        
        
        /// <summary>
        /// cant buy bonus card without resources test
        /// </summary>
        [TestMethod]
        public void CantBuyBonusCardTest()
        {
            model.Board.Players[0].Resources[ResourceType.Grain] = 1;
            model.Board.Players[0].Resources[ResourceType.Wool] = 1;
            model.Board.Players[0].Resources[ResourceType.Ore] = 1;

            Assert.IsTrue(model.Board.Players[0].CanBuyDevelopmentCard);
            Assert.AreEqual(model.Board.BuyCard(model.Board.Players[0]),CardType.None);

            model.Board.Players[0].Resources[ResourceType.Grain] = 0;
            model.Board.Players[0].Resources[ResourceType.Wool] = 1;
            model.Board.Players[0].Resources[ResourceType.Ore] = 1;
            //skip start of the game
            for (int i = 0; i < 6; i++)
            {
                model.StepGame();
            }
            model.Roll();
            Assert.IsFalse(model.Board.Players[0].CanBuyDevelopmentCard);
            Assert.AreEqual(model.Board.BuyCard(model.Board.Players[0]), CardType.None);

            model.Board.Players[0].Resources[ResourceType.Grain] = 1;
            model.Board.Players[0].Resources[ResourceType.Wool] = 0;
            model.Board.Players[0].Resources[ResourceType.Ore] = 1;
            Assert.IsFalse(model.Board.Players[0].CanBuyDevelopmentCard);
            Assert.AreEqual(model.Board.BuyCard(model.Board.Players[0]), CardType.None);

            model.Board.Players[0].Resources[ResourceType.Grain] = 1;
            model.Board.Players[0].Resources[ResourceType.Wool] = 1;
            model.Board.Players[0].Resources[ResourceType.Ore] = 0;
            Assert.IsFalse(model.Board.Players[0].CanBuyDevelopmentCard);
            Assert.AreEqual(model.Board.BuyCard(model.Board.Players[0]), CardType.None);

            model.Board.Players[0].Resources[ResourceType.Grain] = 0;
            model.Board.Players[0].Resources[ResourceType.Wool] = 0;
            model.Board.Players[0].Resources[ResourceType.Ore] = 0;
            Assert.IsFalse(model.Board.Players[0].CanBuyDevelopmentCard);
            Assert.AreEqual(model.Board.BuyCard(model.Board.Players[0]), CardType.None);

            model.Board.Players[0].Resources[ResourceType.Grain] = 0;
            model.Board.Players[0].Resources[ResourceType.Wool] = 0;
            model.Board.Players[0].Resources[ResourceType.Ore] = 1;
            Assert.IsFalse(model.Board.Players[0].CanBuyDevelopmentCard);
            Assert.AreEqual(model.Board.BuyCard(model.Board.Players[0]), CardType.None);

            model.Board.Players[0].Resources[ResourceType.Grain] = 0;
            model.Board.Players[0].Resources[ResourceType.Wool] = 1;
            model.Board.Players[0].Resources[ResourceType.Ore] = 0;
            Assert.IsFalse(model.Board.Players[0].CanBuyDevelopmentCard);
            Assert.AreEqual(model.Board.BuyCard(model.Board.Players[0]), CardType.None);

            model.Board.Players[0].Resources[ResourceType.Grain] = 1;
            model.Board.Players[0].Resources[ResourceType.Wool] = 0;
            model.Board.Players[0].Resources[ResourceType.Ore] = 0;
            Assert.IsFalse(model.Board.Players[0].CanBuyDevelopmentCard);
            Assert.AreEqual(model.Board.BuyCard(model.Board.Players[0]), CardType.None);
        }

        /// <summary>
        /// buy bonus card and gets the victory point or knight test
        /// </summary>
        [TestMethod]
        public void BuyBonusCardVictoryPointAndKnightTest()
        {
            Assert.AreEqual(model.Board.Players[0].DevelopmentCards[(CardType)CardType.Knight], 0);
            Assert.AreEqual(model.Board.Players[0].DevelopmentCards[(CardType)CardType.VictoryPoint], 0);
            CatanBoard.Rand = new RandomMock(new List<int>() { 0, 1 });
            Assert.IsFalse(model.Board.Players[0].CanBuyDevelopmentCard);
            model.Board.Players[0].Resources[ResourceType.Grain] = 2;
            model.Board.Players[0].Resources[ResourceType.Wool] = 2;
            model.Board.Players[0].Resources[ResourceType.Ore] = 2;
            //skip start of the game
            for (int i = 0; i < 6; i++)
            {
                model.StepGame();
            }
            model.Roll();

            Assert.IsTrue(model.Board.Players[0].CanBuyDevelopmentCard);
            Assert.AreEqual(model.Board.BuyCard(model.Board.Players[0]), CardType.Knight);
            Assert.AreEqual(model.Board.Players[0].DevelopmentCards[(CardType)CardType.Knight], 1);
            Assert.AreEqual(model.Board.Players[0].DevelopmentCards[(CardType)CardType.VictoryPoint], 0);
            Assert.IsTrue(model.Board.Players[0].CanBuyDevelopmentCard);
            Assert.AreEqual(model.Board.BuyCard(model.Board.Players[0]), CardType.VictoryPoint);
            Assert.AreEqual(model.Board.Players[0].DevelopmentCards[(CardType)CardType.Knight], 1);
            Assert.AreEqual(model.Board.Players[0].DevelopmentCards[(CardType)CardType.VictoryPoint], 1);
            Assert.AreEqual(model.Board.Players[0].Score, 1);
        }

        /// <summary>
        /// buy bonus card and get the largest army test
        /// </summary>
        [TestMethod]
        public void BuyBonusCardMaxKnightTest()
        {
            Assert.AreEqual(model.Board.Players[0].DevelopmentCards[(CardType)CardType.Knight], 0);
            Assert.AreEqual(model.Board.Players[0].DevelopmentCards[(CardType)CardType.VictoryPoint], 0);
            CatanBoard.Rand = new RandomMock(new List<int>() { 0, 0, 0 });
            Assert.IsFalse(model.Board.Players[0].CanBuyDevelopmentCard);
            model.Board.Players[0].Resources[ResourceType.Grain] = 3;
            model.Board.Players[0].Resources[ResourceType.Wool] = 3;
            model.Board.Players[0].Resources[ResourceType.Ore] = 3;
            //skip start of the game
            for (int i = 0; i < 6; i++)
            {
                model.StepGame();
            }
            model.Roll();
            for (int i = 0; i < 3; i++)
            {
                Assert.IsTrue(model.Board.Players[0].CanBuyDevelopmentCard);
                Assert.AreEqual(model.Board.BuyCard(model.Board.Players[0]), CardType.Knight);
                Assert.AreEqual(model.Board.Players[0].DevelopmentCards[(CardType)CardType.Knight], i+1);
                Assert.AreEqual(model.Board.Players[0].DevelopmentCards[(CardType)CardType.VictoryPoint], 0);
            }
            Assert.AreEqual(model.Board.MaxKnightCardOwnerId, 0);
        }

        /// <summary>
        /// buy bonus card and get the largest army, and the second player get it test
        /// </summary>
        [TestMethod]
        public void BuyBonusCardMaxKnightChangeTest()
        {
            model.Board.Players[0].DevelopmentCards[(CardType)CardType.Knight] = 2;
            Assert.AreEqual(model.Board.Players[0].DevelopmentCards[(CardType)CardType.Knight], 2);
            CatanBoard.Rand = new RandomMock(new List<int>() { 0, 0, 0, 0,0});
            Assert.IsFalse(model.Board.Players[0].CanBuyDevelopmentCard);
            model.Board.Players[0].Resources[ResourceType.Grain] = 1;
            model.Board.Players[0].Resources[ResourceType.Wool] =  1;
            model.Board.Players[0].Resources[ResourceType.Ore]  =  1;
            model.Board.Players[1].Resources[ResourceType.Grain] = 4;
            model.Board.Players[1].Resources[ResourceType.Wool] =  4;
            model.Board.Players[1].Resources[ResourceType.Ore]  =  4;
            //skip start of the game
            for (int i = 0; i < 6; i++)
            {
                model.StepGame();
            }
            model.Roll();
            Assert.IsTrue(model.Board.Players[0].CanBuyDevelopmentCard);
            Assert.AreEqual(model.Board.BuyCard(model.Board.Players[0]), CardType.Knight);
            Assert.AreEqual(model.Board.Players[0].DevelopmentCards[(CardType)CardType.Knight], 3);
            Assert.AreEqual(model.Board.Players[0].DevelopmentCards[(CardType)CardType.VictoryPoint], 0);
            Assert.AreEqual(model.Board.MaxKnightCardOwnerId, 0);
            model.StepGame();
            model.Roll();
            for (int i = 0; i < 3; i++)
            {
                Assert.IsTrue(model.Board.Players[1].CanBuyDevelopmentCard);
                Assert.AreEqual(model.Board.BuyCard(model.Board.Players[1]), CardType.Knight);
                Assert.AreEqual(model.Board.Players[1].DevelopmentCards[(CardType)CardType.Knight], i+1);
                Assert.AreEqual(model.Board.Players[1].DevelopmentCards[(CardType)CardType.VictoryPoint], 0);
                Assert.AreEqual(model.Board.MaxKnightCardOwnerId, 0);
            }
            Assert.IsTrue(model.Board.Players[1].CanBuyDevelopmentCard);
            Assert.AreEqual(model.Board.BuyCard(model.Board.Players[1]), CardType.Knight);
            Assert.AreEqual(model.Board.Players[1].DevelopmentCards[(CardType)CardType.Knight], 4);
            Assert.AreEqual(model.Board.Players[1].DevelopmentCards[(CardType)CardType.VictoryPoint], 0);
            Assert.AreEqual(model.Board.MaxKnightCardOwnerId, 1);
        }

        /// <summary>
        /// unsuccessfull house placement test
        /// </summary>
        [TestMethod]
        public void UnsuccessfullyPlaceHouseTest()
        {
            model.Board.Players[0].Resources[ResourceType.Lumber] = 1;
            model.Board.Players[0].Resources[ResourceType.Grain] = 1;
            model.Board.Players[0].Resources[ResourceType.Wool] = 1;
            model.Board.Players[0].Resources[ResourceType.Brick] = 1;

            Assert.IsTrue(model.Board.PlaceSettlement(new NodeCoordinates(3, 3, Direction.Bottom), model.Board.Players[0]));
            Assert.IsFalse(model.Board.PlaceSettlement(new NodeCoordinates(3, 3, Direction.Bottom), model.Board.Players[0]));

            for (int r = 0; r < model.Board.Size; ++r)
            {
                for (int q = 0; q < model.Board.Size; ++q)
                {
                    for (int dir = 0; dir < 2; ++dir)
                    {
                        if (r == 3 && q == 3 && dir == (int)Direction.Bottom)
                            Assert.AreEqual(HouseType.Settlement, model.Board.Nodes[r, q, (Direction)dir].Building.Type);
                        else
                            Assert.IsTrue(model.Board.Nodes[r, q, (Direction)dir] is null
                                || model.Board.Nodes[r, q, (Direction)dir].Building is null
                                || model.Board.Nodes[r, q, (Direction)dir].Building.Type == HouseType.NoBuilding);

                    }
                }
            }

            Assert.AreEqual(1, model.Board.Players[0].Resources[ResourceType.Lumber]);
            Assert.AreEqual(1, model.Board.Players[0].Resources[ResourceType.Grain]);
            Assert.AreEqual(1, model.Board.Players[0].Resources[ResourceType.Wool]);
            Assert.AreEqual(1, model.Board.Players[0].Resources[ResourceType.Brick]);
        }

        /// <summary>
        /// robber placement test
        /// </summary>
        [TestMethod]
        public void TestRobberPlacement()
        {
            CatanBoard.Rand = new RandomMock(new List<int>() { 3, 4 });

            // skip the start of the game
            Assert.AreEqual(0, model.CurrentPlayerId);
            model.StepGame();
            Assert.AreEqual(1, model.CurrentPlayerId);
            model.StepGame();
            Assert.AreEqual(2, model.CurrentPlayerId);
            model.StepGame();
            Assert.AreEqual(2, model.CurrentPlayerId);
            model.StepGame();
            Assert.AreEqual(1, model.CurrentPlayerId);
            model.StepGame();
            Assert.AreEqual(0, model.CurrentPlayerId);
            model.StepGame();
            Assert.AreEqual(0, model.CurrentPlayerId);
            model.Roll();

            int lastPlayerId = model.CurrentPlayerId;
            model.StepGame();
            Assert.AreEqual(lastPlayerId, model.CurrentPlayerId);

            List<(int R, int Q)> possiblePositions = model.Board.PossibleRobberPositions();
            model.MoveRobber(possiblePositions[0]);

            Assert.AreEqual(possiblePositions[0], model.Board.RobberPosition);
        }

        /// <summary>
        /// unsuccessfull robber placement test
        /// </summary>
        [TestMethod]
        public void TestFailedRobberPlacement()
        {
            CatanBoard.Rand = new RandomMock(new List<int>() { 1, 4 });

            // skip the start of the game
            Assert.AreEqual(0, model.CurrentPlayerId);
            model.StepGame();
            Assert.AreEqual(1, model.CurrentPlayerId);
            model.StepGame();
            Assert.AreEqual(2, model.CurrentPlayerId);
            model.StepGame();
            Assert.AreEqual(2, model.CurrentPlayerId);
            model.StepGame();
            Assert.AreEqual(1, model.CurrentPlayerId);
            model.StepGame();
            Assert.AreEqual(0, model.CurrentPlayerId);
            model.StepGame();
            Assert.AreEqual(0, model.CurrentPlayerId);
            model.Roll();

            int lastPlayerId = model.CurrentPlayerId;
            model.StepGame();
            Assert.AreNotEqual(lastPlayerId, model.CurrentPlayerId);

            List<(int R, int Q)> possiblePositions = model.Board.PossibleRobberPositions();
            (int R, int Q) oldPosition = model.Board.RobberPosition; 
            model.MoveRobber(possiblePositions[0]);

            Assert.AreEqual(oldPosition, model.Board.RobberPosition);
        }

        /// <summary>
        /// robber placement in the same place test
        /// </summary>
        [TestMethod]
        public void RobberPlacementInTheSamePlaceTest()
        {
            CatanBoard.Rand = new RandomMock(new List<int>() { 5, 2 });
            for (int i = 0; i < 6; i++)
                model.StepGame();
            model.Roll();

            model.MoveRobber(model.Board.RobberPosition);

            int lastPlayerId = model.CurrentPlayerId;
            model.StepGame();

            Assert.AreEqual(lastPlayerId, model.CurrentPlayerId);
        }

        /// <summary>
        /// Initial Turn Robber Placement Test
        /// </summary>
        [TestMethod]
        public void InitialTurnRobberPlacementTest()
        {
            CatanBoard.Rand = new RandomMock(new List<int>() { 1, 6 });

            List<(int R, int Q)> possiblePositions = model.Board.PossibleRobberPositions();
            model.MoveRobber(possiblePositions[5]);

            Assert.AreNotEqual(possiblePositions[5], model.Board.RobberPosition);
        }

        /// <summary>
        /// longest road test
        /// </summary>
        [TestMethod]
        public void LongestRoadTest()
        {
            model.UpgradeNode(new NodeCoordinates(3, 3, Direction.Top), 0);
            model.PlaceRoad((GetNodeId(2, 4, Direction.Bottom), GetNodeId(3, 3, Direction.Top)));
            
            for (int i = 0; i < 6; i++)
                model.StepGame();
            model.Board.Players[0].Resources[ResourceType.Lumber] = 100;
            model.Board.Players[0].Resources[ResourceType.Brick] = 100;
            model.Board.Players[0].Resources[ResourceType.Grain] = 100;
            model.Board.Players[0].Resources[ResourceType.Ore] = 100;
            model.Board.Players[0].Resources[ResourceType.Wool] = 100;
            
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(2, 4, Direction.Bottom), new NodeCoordinates(3, 3, Direction.Top))));
            Assert.AreNotEqual(model.Board.LongestRoadOwnerId, 0);
            model.PlaceRoad((GetNodeId(2, 4, Direction.Bottom), GetNodeId(4, 3, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(2, 4, Direction.Bottom), new NodeCoordinates(4, 3, Direction.Top))));
            Assert.AreNotEqual(model.Board.LongestRoadOwnerId, 0);
            model.PlaceRoad((GetNodeId(3, 3, Direction.Bottom), GetNodeId(4, 3, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(3, 3, Direction.Bottom), new NodeCoordinates(4, 3, Direction.Top))));
            Assert.AreNotEqual(model.Board.LongestRoadOwnerId, 0);
            model.PlaceRoad((GetNodeId(3, 3, Direction.Bottom), GetNodeId(4, 2, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(3, 3, Direction.Bottom), new NodeCoordinates(4, 2, Direction.Top))));
            Assert.AreNotEqual(model.Board.LongestRoadOwnerId, 0);
            model.PlaceRoad((GetNodeId(2, 3, Direction.Bottom), GetNodeId(4, 2, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(2, 3, Direction.Bottom), new NodeCoordinates(4, 2, Direction.Top))));
            Assert.AreEqual(model.Board.LongestRoadOwnerId, 0);
            Assert.AreEqual(model.Board.MaxRoadLength, 5);
        }

        /// <summary>
        /// longest road with circle test
        /// </summary>
        [TestMethod]
        public void LongestRoadWithCircleTest()
        {
            model.UpgradeNode(new NodeCoordinates(3, 3, Direction.Top), 0);
            model.PlaceRoad((GetNodeId(2, 4, Direction.Bottom), GetNodeId(3, 3, Direction.Top)));

            for (int i = 0; i < 6; i++)
                model.StepGame();
            model.Board.Players[0].Resources[ResourceType.Lumber] = 100;
            model.Board.Players[0].Resources[ResourceType.Brick] = 100;
            model.Board.Players[0].Resources[ResourceType.Grain] = 100;
            model.Board.Players[0].Resources[ResourceType.Ore] = 100;
            model.Board.Players[0].Resources[ResourceType.Wool] = 100;

            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(2, 4, Direction.Bottom), new NodeCoordinates(3, 3, Direction.Top))));
            Assert.AreNotEqual(model.Board.LongestRoadOwnerId, 0);
            model.PlaceRoad((GetNodeId(2, 4, Direction.Bottom), GetNodeId(4, 3, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(2, 4, Direction.Bottom), new NodeCoordinates(4, 3, Direction.Top))));
            Assert.AreNotEqual(model.Board.LongestRoadOwnerId, 0);
            model.PlaceRoad((GetNodeId(3, 3, Direction.Bottom), GetNodeId(4, 3, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(3, 3, Direction.Bottom), new NodeCoordinates(4, 3, Direction.Top))));
            Assert.AreNotEqual(model.Board.LongestRoadOwnerId, 0);
            model.PlaceRoad((GetNodeId(3, 3, Direction.Bottom), GetNodeId(4, 2, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(3, 3, Direction.Bottom), new NodeCoordinates(4, 2, Direction.Top))));
            Assert.AreNotEqual(model.Board.LongestRoadOwnerId, 0);
            model.PlaceRoad((GetNodeId(2, 3, Direction.Bottom), GetNodeId(4, 2, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(2, 3, Direction.Bottom), new NodeCoordinates(4, 2, Direction.Top))));
            Assert.AreEqual(model.Board.LongestRoadOwnerId, 0);
            Assert.AreEqual(model.Board.MaxRoadLength, 5);
            model.PlaceRoad((GetNodeId(2, 3, Direction.Bottom), GetNodeId(3, 3, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(2, 3, Direction.Bottom), new NodeCoordinates(3, 3, Direction.Top))));
            Assert.AreEqual(model.Board.LongestRoadOwnerId, 0);
            Assert.AreEqual(model.Board.MaxRoadLength, 6);
        }

        /// <summary>
        /// another longest road with circle test
        /// </summary>
        [TestMethod]
        public void LongestRoadWithCircleTest2()
        {
            model.UpgradeNode(new NodeCoordinates(3, 3, Direction.Top), 0);
            model.PlaceRoad((GetNodeId(2, 4, Direction.Bottom), GetNodeId(3, 3, Direction.Top)));

            for (int i = 0; i < 6; i++)
                model.StepGame();
            model.Board.Players[0].Resources[ResourceType.Lumber] = 100;
            model.Board.Players[0].Resources[ResourceType.Brick] = 100;
            model.Board.Players[0].Resources[ResourceType.Grain] = 100;
            model.Board.Players[0].Resources[ResourceType.Ore] = 100;
            model.Board.Players[0].Resources[ResourceType.Wool] = 100;

            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(2, 4, Direction.Bottom), new NodeCoordinates(3, 3, Direction.Top))));
            Assert.AreNotEqual(model.Board.LongestRoadOwnerId, 0);
            model.PlaceRoad((GetNodeId(2, 4, Direction.Bottom), GetNodeId(4, 3, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(2, 4, Direction.Bottom), new NodeCoordinates(4, 3, Direction.Top))));
            Assert.AreNotEqual(model.Board.LongestRoadOwnerId, 0);
            model.PlaceRoad((GetNodeId(3, 3, Direction.Bottom), GetNodeId(4, 3, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(3, 3, Direction.Bottom), new NodeCoordinates(4, 3, Direction.Top))));
            Assert.AreNotEqual(model.Board.LongestRoadOwnerId, 0);
            model.PlaceRoad((GetNodeId(3, 3, Direction.Bottom), GetNodeId(4, 2, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(3, 3, Direction.Bottom), new NodeCoordinates(4, 2, Direction.Top))));
            Assert.AreNotEqual(model.Board.LongestRoadOwnerId, 0);
            model.PlaceRoad((GetNodeId(2, 3, Direction.Bottom), GetNodeId(4, 2, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(2, 3, Direction.Bottom), new NodeCoordinates(4, 2, Direction.Top))));
            Assert.AreEqual(model.Board.LongestRoadOwnerId, 0);
            Assert.AreEqual(model.Board.MaxRoadLength, 5);
            model.PlaceRoad((GetNodeId(2, 3, Direction.Bottom), GetNodeId(3, 3, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(2, 3, Direction.Bottom), new NodeCoordinates(3, 3, Direction.Top))));
            Assert.AreEqual(model.Board.LongestRoadOwnerId, 0);
            Assert.AreEqual(model.Board.MaxRoadLength, 6);
            model.PlaceRoad((GetNodeId(4, 3, Direction.Top), GetNodeId(3, 4, Direction.Bottom)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(4, 3, Direction.Top), new NodeCoordinates(3, 4, Direction.Bottom))));
            Assert.AreEqual(model.Board.LongestRoadOwnerId, 0);
            Assert.AreEqual(model.Board.MaxRoadLength, 7);
            model.PlaceRoad((GetNodeId(4, 4, Direction.Top), GetNodeId(3, 4, Direction.Bottom)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(4, 4, Direction.Top), new NodeCoordinates(3, 4, Direction.Bottom))));
            Assert.AreEqual(model.Board.LongestRoadOwnerId, 0);
            Assert.AreEqual(model.Board.MaxRoadLength, 8);
        }

        /// <summary>
        /// another longest road with circle test
        /// </summary>
        [TestMethod]
        public void LongestRoadWithCircleTest3()
        {
            model.UpgradeNode(new NodeCoordinates(3, 3, Direction.Top), 0);
            model.PlaceRoad((GetNodeId(2, 4, Direction.Bottom), GetNodeId(3, 3, Direction.Top)));

            for (int i = 0; i < 6; i++)
                model.StepGame();
            model.Board.Players[0].Resources[ResourceType.Lumber] = 100;
            model.Board.Players[0].Resources[ResourceType.Brick] = 100;
            model.Board.Players[0].Resources[ResourceType.Grain] = 100;
            model.Board.Players[0].Resources[ResourceType.Ore] = 100;
            model.Board.Players[0].Resources[ResourceType.Wool] = 100;

            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(2, 4, Direction.Bottom), new NodeCoordinates(3, 3, Direction.Top))));
            Assert.AreNotEqual(model.Board.LongestRoadOwnerId, 0);
            model.PlaceRoad((GetNodeId(2, 4, Direction.Bottom), GetNodeId(4, 3, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(2, 4, Direction.Bottom), new NodeCoordinates(4, 3, Direction.Top))));
            Assert.AreNotEqual(model.Board.LongestRoadOwnerId, 0);
            model.PlaceRoad((GetNodeId(3, 3, Direction.Bottom), GetNodeId(4, 3, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(3, 3, Direction.Bottom), new NodeCoordinates(4, 3, Direction.Top))));
            Assert.AreNotEqual(model.Board.LongestRoadOwnerId, 0);
            model.PlaceRoad((GetNodeId(3, 3, Direction.Bottom), GetNodeId(4, 2, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(3, 3, Direction.Bottom), new NodeCoordinates(4, 2, Direction.Top))));
            Assert.AreNotEqual(model.Board.LongestRoadOwnerId, 0);
            model.PlaceRoad((GetNodeId(2, 3, Direction.Bottom), GetNodeId(4, 2, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(2, 3, Direction.Bottom), new NodeCoordinates(4, 2, Direction.Top))));
            Assert.AreEqual(model.Board.LongestRoadOwnerId, 0);
            Assert.AreEqual(model.Board.MaxRoadLength, 5);
            model.PlaceRoad((GetNodeId(2, 3, Direction.Bottom), GetNodeId(3, 3, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(2, 3, Direction.Bottom), new NodeCoordinates(3, 3, Direction.Top))));
            Assert.AreEqual(model.Board.LongestRoadOwnerId, 0);
            Assert.AreEqual(model.Board.MaxRoadLength, 6);
            model.PlaceRoad((GetNodeId(4, 3, Direction.Top), GetNodeId(3, 4, Direction.Bottom)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(4, 3, Direction.Top), new NodeCoordinates(3, 4, Direction.Bottom))));
            Assert.AreEqual(model.Board.LongestRoadOwnerId, 0);
            Assert.AreEqual(model.Board.MaxRoadLength, 7);
            model.PlaceRoad((GetNodeId(4, 4, Direction.Top), GetNodeId(3, 4, Direction.Bottom)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(4, 4, Direction.Top), new NodeCoordinates(3, 4, Direction.Bottom))));
            Assert.AreEqual(model.Board.LongestRoadOwnerId, 0);
            Assert.AreEqual(model.Board.MaxRoadLength, 8);
            model.PlaceRoad((GetNodeId(3, 3, Direction.Top), GetNodeId(1, 4, Direction.Bottom)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(3, 3, Direction.Top), new NodeCoordinates(1, 4, Direction.Bottom))));
            Assert.AreEqual(model.Board.LongestRoadOwnerId, 0);
            Assert.AreEqual(model.Board.MaxRoadLength, 8);
            model.PlaceRoad((GetNodeId(2, 3, Direction.Top), GetNodeId(1, 4, Direction.Bottom)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(2, 3, Direction.Top), new NodeCoordinates(1, 4, Direction.Bottom))));
            Assert.AreEqual(model.Board.LongestRoadOwnerId, 0);
            Assert.AreEqual(model.Board.MaxRoadLength, 8);
            model.PlaceRoad((GetNodeId(2, 3, Direction.Top), GetNodeId(1, 3, Direction.Bottom)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(2, 3, Direction.Top), new NodeCoordinates(1, 3, Direction.Bottom))));
            Assert.AreEqual(model.Board.LongestRoadOwnerId, 0);
            Assert.AreEqual(model.Board.MaxRoadLength, 9);
        }

        /// <summary>
        /// take over the longest road test
        /// </summary>
        [TestMethod]
        public void LongestRoadTakeOverTest()
        {
            CatanBoard.Rand = new RandomMock(new List<int>() { 3, 2 });

            model.Board.Players[0].Resources[ResourceType.Lumber] = 100;
            model.Board.Players[0].Resources[ResourceType.Brick] = 100;
            model.Board.Players[0].Resources[ResourceType.Grain] = 100;
            model.Board.Players[0].Resources[ResourceType.Ore] = 100;
            model.Board.Players[0].Resources[ResourceType.Wool] = 100;
            model.Board.Players[1].Resources[ResourceType.Lumber] = 100;
            model.Board.Players[1].Resources[ResourceType.Brick] = 100;
            model.Board.Players[1].Resources[ResourceType.Grain] = 100;
            model.Board.Players[1].Resources[ResourceType.Ore] = 100;
            model.Board.Players[1].Resources[ResourceType.Wool] = 100;

            model.UpgradeNode(new NodeCoordinates(3, 3, Direction.Top), 0);
            model.PlaceRoad((GetNodeId(2, 4, Direction.Bottom), GetNodeId(3, 3, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(2, 4, Direction.Bottom), new NodeCoordinates(3, 3, Direction.Top))));
            //model.StepGame();
            model.UpgradeNode(new NodeCoordinates(4, 4, Direction.Top), 1);
            Assert.AreEqual(model.CurrentPlayerId, 1);
            model.PlaceRoad((GetNodeId(3, 5, Direction.Bottom), GetNodeId(4, 4, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(3, 5, Direction.Bottom), new NodeCoordinates(4, 4, Direction.Top))));

            for (int i = 0; i < 4; i++)
                model.StepGame();
            Assert.AreEqual(model.CurrentPlayerId, 0);

            Assert.AreNotEqual(model.Board.LongestRoadOwnerId, 0);
            model.PlaceRoad((GetNodeId(2, 4, Direction.Bottom), GetNodeId(4, 3, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(2, 4, Direction.Bottom), new NodeCoordinates(4, 3, Direction.Top))));
            Assert.AreNotEqual(model.Board.LongestRoadOwnerId, 0);
            model.PlaceRoad((GetNodeId(3, 3, Direction.Bottom), GetNodeId(4, 3, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(3, 3, Direction.Bottom), new NodeCoordinates(4, 3, Direction.Top))));
            Assert.AreNotEqual(model.Board.LongestRoadOwnerId, 0);
            model.PlaceRoad((GetNodeId(3, 3, Direction.Bottom), GetNodeId(4, 2, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(3, 3, Direction.Bottom), new NodeCoordinates(4, 2, Direction.Top))));
            Assert.AreNotEqual(model.Board.LongestRoadOwnerId, 0);
            model.PlaceRoad((GetNodeId(2, 3, Direction.Bottom), GetNodeId(4, 2, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(2, 3, Direction.Bottom), new NodeCoordinates(4, 2, Direction.Top))));
            Assert.AreEqual(model.Board.LongestRoadOwnerId, 0);
            Assert.AreEqual(model.Board.MaxRoadLength, 5);

            model.StepGame();
            model.Roll();
            model.StepGame();
            model.Roll();

            Assert.AreEqual(model.CurrentPlayerId, 1);
            Assert.AreEqual(model.Board.LongestRoadOwnerId, 0);
            model.PlaceRoad((GetNodeId(3, 5, Direction.Bottom), GetNodeId(5, 4, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(3, 5, Direction.Bottom), new NodeCoordinates(5, 4, Direction.Top))));
            Assert.AreEqual(model.Board.LongestRoadOwnerId, 0);
            model.PlaceRoad((GetNodeId(4, 4, Direction.Bottom), GetNodeId(5, 4, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(4, 4, Direction.Bottom), new NodeCoordinates(5, 4, Direction.Top))));
            Assert.AreEqual(model.Board.LongestRoadOwnerId, 0);
            model.PlaceRoad((GetNodeId(4, 4, Direction.Bottom), GetNodeId(5, 3, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(4, 4, Direction.Bottom), new NodeCoordinates(5, 3, Direction.Top))));
            Assert.AreEqual(model.Board.LongestRoadOwnerId, 0);
            model.PlaceRoad((GetNodeId(3, 4, Direction.Bottom), GetNodeId(5, 3, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(3, 4, Direction.Bottom), new NodeCoordinates(5, 3, Direction.Top))));
            Assert.AreEqual(model.Board.LongestRoadOwnerId, 0);
            Assert.AreEqual(model.Board.MaxRoadLength, 5);
            model.PlaceRoad((GetNodeId(3, 4, Direction.Bottom), GetNodeId(4, 4, Direction.Top)));
            Assert.IsTrue(model.Board.Roads.IsPlacedRoad(new RoadCoordinates(new NodeCoordinates(3, 4, Direction.Bottom), new NodeCoordinates(4, 4, Direction.Top))));
            Assert.AreEqual(model.Board.LongestRoadOwnerId, 1);
            Assert.AreEqual(model.Board.MaxRoadLength, 6);
        }
    }
}
