﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CatanGame.Test
{
    /// <summary>
    /// Mocks the random class.
    /// </summary>
    internal class RandomMock : Random
    {
        private int _currId = 0;
        private List<int> _customValues;

        public RandomMock(List<int> customValues)
        {
            if (customValues is null || customValues.Count == 0)
                throw new InvalidOperationException("list can't be empty or null!");
            _customValues = customValues;
        }

        /// <summary>
        /// Gets the next mocked value.
        /// </summary>
        /// <returns>The next mocked value.</returns>
        public override int Next()
        {
            int result = _customValues[_currId];
            _currId = (_currId + 1) % _customValues.Count;
            return result;
        }

        /// <summary>
        /// Gets the next mocked value.
        /// </summary>
        /// <param name="maxValue">The maximum value.</param>
        /// <returns>The next mocked value.</returns>
        public override int Next(int maxValue)
        {
            return Math.Min(Next(), maxValue - 1);
        }

        /// <summary>
        /// Gets the next mocked value.
        /// </summary>
        /// <param name="minValue">The minimum value.</param>
        /// <param name="maxValue">The maximum value.</param>
        /// <returns>The next mocked value.</returns>
        public override int Next(int minValue, int maxValue)
        {
            return Math.Clamp(minValue, Next(), maxValue - 1);
        }

        // This method is not implemented.
        public override void NextBytes(byte[] buffer)
        {
            throw new NotImplementedException();
        }

        // This method is not implemented.
        public override void NextBytes(Span<byte> buffer)
        {
            throw new NotImplementedException();
        }

        // This method is not implemented.
        public override double NextDouble()
        {
            throw new NotImplementedException();
        }

        // This method is not implemented.
        protected override double Sample()
        {
            throw new NotImplementedException();
        }
    }
}
