﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Controls;
using System.Windows.Shapes;
using System.Windows.Media;

namespace CatanGame.View
{
    /// <summary>
    /// A field on the board.
    /// </summary>
    public class CatanField
    {
        /// <summary>
        /// Gets or sets the Polygon representing the field.
        /// </summary>
        public Polygon Field { get; set; }
        /// <summary>
        /// Gets or sets the text in the field.
        /// </summary>
        public Label Text { get; set; }

        /// <summary>
        /// Gets or sets the robber on the field.
        /// </summary>
        public Ellipse Robber { get; set; }

        public CatanField(Point position, (int R, int Q) coords, double radius)
        {
            Field = HexagonFactory.MakeHexagon(position, radius);
            Text = new Label();
            Text.Loaded += (obj, args) 
                => Text.Margin = new Thickness(position.X - Text.ActualWidth / 2, position.Y - Text.ActualHeight / 2, 0, 0);
            Robber = new Ellipse();
            Robber.Loaded += (obj, args)
                => Robber.Margin = new Thickness(position.X - Robber.ActualWidth / 2, Text.Margin.Top - Robber.ActualHeight / 2, 0, 0);
        }

        /// <summary>
        /// Updates the position of the field.
        /// </summary>
        /// <param name="position">The position relative to the canvas.</param>
        /// <param name="radius">The radius of the hexagon.</param>
        public void Update(Point position, double radius)
        {
            Field.Points = HexagonFactory.GetHexagonCorners(position, radius);
            Text.Margin = new Thickness(position.X - Text.ActualWidth / 2, position.Y - Text.ActualHeight / 2, 0, 0);
            Robber.Margin = new Thickness(position.X - Robber.ActualWidth / 2, Text.Margin.Top - Robber.ActualHeight / 2, 0, 0);
        }
    }
}
