﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Media;
using System.Windows.Shapes;

namespace CatanGame.View
{
    /// <summary>
    /// Creates hexagons.
    /// </summary>
    class HexagonFactory
    {
        private static readonly int cornerCount = 6;
        private static double sqrt3 = Math.Sqrt(3);
        private static double sqrt3div2 = sqrt3 / 2;
        private static double[] idToCos = new double[]{ 
            sqrt3div2, sqrt3div2, 0, -sqrt3div2, -sqrt3div2, 0
        };
        private static double[] idToSin = new double[] {
            -0.5, 0.5, 1, 0.5, -0.5, -1, 
        };

        /// <summary>
        /// Makes a hexagon with given size at given position.
        /// </summary>
        /// <param name="center">The center coordinates of the hexagon.</param>
        /// <param name="size">The size of the hexagon.</param>
        /// <returns>The new hexagon.</returns>
        public static Polygon MakeHexagon(Point center, double size)
        {
            return new Polygon()
            {
                Points = GetHexagonCorners(center, size)
            };
        }

        /// <summary>
        /// Gets the corners of the hexagon.
        /// </summary>
        /// <param name="center">The center coordinate of the hexagon.</param>
        /// <param name="size">The size of the hexagon.</param>
        /// <returns>The corners of the hexagon.</returns>
        public static PointCollection GetHexagonCorners(Point center, double size)
        {
            PointCollection hexagonCorners = new PointCollection();
            for (int i = 0; i < cornerCount; i++)
                hexagonCorners.Add(HexCorner(center, size, i));

            return hexagonCorners;
        }





        /// <summary>
        /// Gets the position of the hexagon corner at given id.
        /// </summary>
        /// <param name="center">The center coordinate of the hexagon.</param>
        /// <param name="size">The size of the hexagon.</param>
        /// <param name="i">The id of the corner of the hexagon, where 0 <= i < 6. </param>
        /// <returns></returns>
        public static Point HexCorner(Point center, double size, int i)
        {
            return new Point(center.X + size * idToCos[i],
                             center.Y + size * idToSin[i]);
        }
    }
}
