﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CatanGame.ViewModel
{
    public class TradeDataViewModel : ViewModelBase, IEquatable<TradeDataViewModel>
    {
        private int _currentCnt;
        private int _offeredCnt;
        private int _requestedCnt;
        private bool _canOffer;
        private static readonly int _offerAmount = 3;
        
        /// <summary>
        /// gets or sets the current count
        /// </summary>
        public int CurrentCnt 
        { 
            get => _currentCnt; 
            set
            {
                ChangeProperty(ref _currentCnt, value);
                CanOffer = _offeredCnt + _offerAmount <= _currentCnt;
                TradeDataChanged?.Invoke(this, EventArgs.Empty);
            } 
        }
        /// <summary>
        /// gets or sets the offered count
        /// </summary>
        public int OfferedCnt
        {
            get => _offeredCnt;
            set
            {
                ChangeProperty(ref _offeredCnt, value);
                CanOffer = _offeredCnt + _offerAmount <= _currentCnt;
                TradeDataChanged?.Invoke(this, EventArgs.Empty);
            }
        }
        /// <summary>
        /// gets or sets the requested count
        /// </summary>
        public int RequestedCnt
        {
            get => _requestedCnt;
            set
            {
                ChangeProperty(ref _requestedCnt, value);
                TradeDataChanged?.Invoke(this, EventArgs.Empty);
            }
        }
        /// <summary>
        /// gets or sets the current player can offer
        /// </summary>
        public bool CanOffer
        {
            get => _canOffer;
            set => ChangeProperty(ref _canOffer, value);
        }

        /// <summary>
        /// command for request resorce
        /// </summary>
        public DelegateCommand RequestResourceCommand { get; private set; }
        /// <summary>
        /// command for Offer Resource
        /// </summary>
        public DelegateCommand OfferResourceCommand { get; private set; }

        /// <summary>
        /// event handler of trade data
        /// </summary>
        public event EventHandler TradeDataChanged;

        /// <summary>
        /// trade tara in view model
        /// </summary>
        /// <param name="currentCnt">current resorce count</param>
        /// <param name="offeredCnt">offered resorce count</param>
        /// <param name="requestedCnt">requested resorce count</param>
        public TradeDataViewModel(int currentCnt, int offeredCnt, int requestedCnt)
        {
            CurrentCnt = currentCnt;
            OfferedCnt = offeredCnt;
            RequestedCnt = requestedCnt;

            RequestResourceCommand = new DelegateCommand(key => RequestedCnt++);
            OfferResourceCommand = new DelegateCommand(key => OfferedCnt += _offerAmount);
        }

        /// <summary>
        /// chek the this object equal that other object 
        /// </summary>
        /// <param name="other">other</param>
        /// <returns></returns>
        public bool Equals(TradeDataViewModel other)
        {
            return _currentCnt == other._currentCnt
                   && _offeredCnt == other._offeredCnt
                   && _requestedCnt == other._requestedCnt;
        }
    }
}
