﻿using System;
using System.ComponentModel;
using System.Runtime.CompilerServices;

namespace CatanGame.ViewModel
{
    // Code taken from class resources
    public abstract class ViewModelBase : INotifyPropertyChanged
    {
        /// <summary>
        /// Instance view model primary class.
        /// </summary>
        protected ViewModelBase() { }

        /// <summary>
        /// Tulajdonság változásának eseménye.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;


        /// <summary>
        /// Tproperty changing method
        /// </summary>
        /// <typeparam name="T">A property type</typeparam>
        /// <param name="property">Reference representing the value of the property</param>
        /// <param name="value">the new value</param>
        /// <param name="propertyName">the property name</param>
        public void ChangeProperty<T>(ref T property, T value, [CallerMemberName] String propertyName = null) where T : IEquatable<T>
        {
            if (property is not null && property.Equals(value))
                return;

            property = value;
            OnPropertyChanged(propertyName);
        }

        /// <summary>
        /// Property change with control.
        /// </summary>
        /// <param name="propertyName">the property name.</param>
        protected virtual void OnPropertyChanged([CallerMemberName] String propertyName = null)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
    }
}
